package net.indiespot.sql;

import net.indiespot.sql.Sql.Txn;
import net.indiespot.sql.SqlPool.UnreachableDatabase;
import nl.west.rme.common.util.WebContextThreadLocal;

import java.util.NoSuchElementException;

public class SqlContext {
	private static final WebContextThreadLocal<Sql> THREAD_LOCAL = new WebContextThreadLocal<>();

	public static Sql current() {
		final Sql current = THREAD_LOCAL.get();
		if (current == null)
			throw new NoSuchElementException("missing threadlocal SQL connection");
		return current;
	}

	public static <T> T txn(Sql conn, Txn<T> txn) throws UnreachableDatabase {
		if (THREAD_LOCAL.get() != null)
			throw new IllegalStateException();

		try {
			THREAD_LOCAL.set(conn);
			return conn.txn(txn);
		} finally {
			THREAD_LOCAL.remove();
		}
	}

	public static <T> T txn(SqlPool pool, Txn<T> txn, long timeout) throws UnreachableDatabase {
		if (THREAD_LOCAL.get() != null)
			throw new IllegalStateException();

		final Sql conn = pool.acquire(timeout);
		try {
			THREAD_LOCAL.set(conn);
			return conn.txn(txn);
		} finally {
			THREAD_LOCAL.remove();
			pool.release(conn);
		}
	}
}
