package net.indiespot.sql;

import net.indiespot.sql.SqlPool.UnreachableDatabase;
import nl.west.rme.common.FilterParamUtils;
import nl.west.rme.common.util.RegexRules;
import log.Log;
import java.io.IOException;
import java.sql.SQLException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class SqlPoolFilter implements Filter
{
  private SqlPool pool;
  private RegexRules uriMatcher;

  @Override
  public void init(FilterConfig config)
      throws ServletException
  {
    final int poolSize = FilterParamUtils.parseInt(config, "pool-size", 10);

    uriMatcher = FilterParamUtils.uriRules(config, "uri-rules", null);

    final SqlConnector connector =
      FilterParamUtils.instantiateFQCN(config, "connector-fqcn");
    final SqlConfigurator configurator =
      FilterParamUtils.instantiateFQCN(config, "configurator-fqcn");
    pool = new SqlPool(poolSize, connector, configurator);
  }

  @Override
  public void destroy()
  {
    pool.destroy();
  }

  @Override
  public void doFilter(
      final ServletRequest request, final ServletResponse response,
      final FilterChain chain)
      throws IOException, ServletException
  {
    if (uriMatcher != null
        && !uriMatcher.matchesURI((HttpServletRequest) request)) {
      chain.doFilter(request, response);
      return;
    }

    final Sql.Txn<Void> txn = new Sql.Txn<Void>()
    {
      @Override
      public Void exec(Sql conn)
          throws SQLException
      {
        try {
          chain.doFilter(request, response);

          return null;
        }
        catch (final RuntimeException exc) {
          throw exc;
        }
        catch (final Throwable exc) {
          throw new IllegalStateException(exc);
        }
      }
    };

    try {
      SqlContext.txn(pool, txn, 10_000);
    }
    catch (final UnreachableDatabase exc) {
      Log.error("failed to acquire sql connection", exc);

      ((HttpServletResponse) response)
          .setStatus(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
      response.setContentType("text/html");
      response.getWriter().println("<h1>Service Unavailable</h1>");
      response.getWriter().println("One moment please: " + exc.getMessage());
    }
  }
}
