package nl.billinghouse.rme;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.concurrent.atomic.AtomicInteger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.sql.DataSource;

import net.indiespot.sql.Sql;
import net.indiespot.sql.SqlContext;
import net.indiespot.sql.data.Fetcher;
import net.indiespot.sql.data.Record;
import net.indiespot.sql.data.RecordManagerDomain;
import net.indiespot.sql.dialects.MySqlConnectionConfiguration;
import nl.billinghouse.rme.dao.Build;
import nl.billinghouse.rme.dao.Portal;
import nl.west.rme.common.UriRulesFilter;

public class SqlContextFilter extends UriRulesFilter {
	private DataSource dataSource;
	private final AtomicInteger id_gen = new AtomicInteger();

	@Override
	public void init(FilterConfig config) throws ServletException {
		super.init(config);

		try {
			Context initCtx = new InitialContext();
			Context envCtx = (Context) initCtx.lookup("java:comp/env");
			dataSource = (DataSource) envCtx.lookup("jdbc/essent-manui");
		} catch (Exception exc) {
			exc.printStackTrace();
		}

		RecordManagerDomain.Access.register(new RecordManagerDomain() {
			@Override
			public <T extends Record<T>> Class<T> lookup(String simpleName) {
				if (simpleName.equals("Build"))
					return (Class<T>) Build.class;
				if (simpleName.equals("Portal"))
					return (Class<T>) Portal.class;
				return null;
			}

			@Override
			public <T extends Record<T>> T newRecord(Class<T> type) {
				return null;
			}

			@Override
			public void resolveInboundForeignKeys(Fetcher f) {

				for (Build build : f.getEntities(Build.class))
					build.$meta = new Build.Meta();
				for (Portal portal : f.getEntities(Portal.class))
					portal.$meta = new Portal.Meta();

				for (Portal portal : f.getEntities(Portal.class))
					if (portal.build != null && portal.build.isResolved())
						portal.build.value().$meta.portals.add(portal);

			}
		});
	}

	@Override
	public void doFilterImpl(final ServletRequest req, final ServletResponse resp, final FilterChain chain)
			throws IOException, ServletException {
		try {
			Connection conn = dataSource.getConnection();
			MySqlConnectionConfiguration.configure(conn, false);
			Sql sql = new Sql(conn, id_gen.incrementAndGet());

			try {
				SqlContext.txn(sql, new Sql.Txn<Void>() {
					@Override
					public Void exec(Sql conn) throws SQLException {
						try {
							chain.doFilter(req, resp);
							return null;
						} catch (IOException | ServletException e) {
							throw new IllegalStateException(e);
						}
					}
				});
			} finally {
				sql.close(null);
			}
		} catch (Exception exc) {
			exc.printStackTrace();
		}
	}
}
