package nl.billinghouse.rme.utility;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import log.Log;

@SuppressWarnings("serial")
public class BaseServlet extends HttpServlet {
	@Override
	protected void service(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		Log.info("REQUEST: " + req.getMethod() + " " + req.getRequestURI());

		final Method method = this.getServiceMethod(req);
		if (method == null) {
			resp.sendError(HttpServletResponse.SC_NOT_FOUND);
			return;
		}

		this.onBeforeService(req, resp);

		Object result = null;
		Throwable cause = null;
		try {
			result = method.invoke(this, this.getMethodArguments(req, resp));

			resp.setStatus(HttpServletResponse.SC_OK);
			this.onSuccessHandler(result, req, resp);
		} catch (final IllegalAccessException exc) {
			throw new IllegalStateException(exc);
		} catch (final InvocationTargetException exc) {
			cause = exc.getCause();
			if (cause == null)
				throw new IllegalStateException();

			resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
			this.onServiceFailure(cause, req, resp);
		} finally {
			this.onAfterService(result, cause, req, resp);
		}
	}

	private Method getServiceMethod(HttpServletRequest req) {
		try {
			return this.getClass().getMethod(this.getMethodName(req), this.getMethodParameters());
		} catch (final NoSuchMethodException exc1) {
			try {
				return this.getClass().getMethod(this.getDefaultMethodName(req), this.getMethodParameters());
			} catch (final NoSuchMethodException exc2) {
				return null;
			}
		}
	}

	protected static String[] getPath(HttpServletRequest req) {
		final String contextPath = req.getContextPath();
		final String servletPath = req.getServletPath();
		final String requestURI = req.getRequestURI();
		String relativePath = requestURI;

		if (!relativePath.startsWith(contextPath))
			throw new IllegalStateException();
		relativePath = relativePath.substring(contextPath.length());

		if (!relativePath.startsWith(servletPath))
			throw new IllegalStateException();
		relativePath = relativePath.substring(servletPath.length());

		while (relativePath.startsWith("/"))
			relativePath = relativePath.substring(1);

		return relativePath.split("/");
	}

	protected String getMethodName(HttpServletRequest req) {
		final String[] path = getPath(req);
		if (path[0].isEmpty())
			path[0] = "index";

		// do[method][relative-path]
		// POST /fancy-operation/launch/unicorn -->
		// "doPostFancyOperation(req, resp)"
		final String methodName = "do" + Text.capitalize(req.getMethod().toLowerCase());

		final String[] x = path[0].split("-");
		for (int i = 0; i < x.length; i++)
			x[i] = Text.capitalize(x[i].toLowerCase());
		return methodName + Text.join("", x);
	}

	protected String getDefaultMethodName(HttpServletRequest req) {
		final String methodName = "do" + Text.capitalize(req.getMethod().toLowerCase());
		return methodName;
	}

	protected Class<?>[] getMethodParameters() {
		return new Class[] { HttpServletRequest.class, HttpServletResponse.class };
	}

	protected Object[] getMethodArguments(HttpServletRequest req, HttpServletResponse resp) {
		return new Object[] { req, resp };
	}

	/*
	 * handlers
	 */

	protected void onBeforeService(HttpServletRequest req, HttpServletResponse resp) {

	}

	protected void onSuccessHandler(Object result, HttpServletRequest req, HttpServletResponse resp)
			throws IOException {

	}

	protected void onServiceFailure(Throwable cause, HttpServletRequest req, HttpServletResponse resp)
			throws IOException {
		resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);

		cause.printStackTrace();
	}

	protected void onAfterService(Object result, Throwable cause, HttpServletRequest req, HttpServletResponse resp)
			throws IOException {

	}
}
