package nl.billinghouse.rme.utility;

import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Collection;


public class Text
{
  public static String capitalize(String s)
  {
    if (s.isEmpty())
      return s;
    return Character.toUpperCase(s.charAt(0)) + s.substring(1);
  }

  public static String join(String sep, String... arr)
  {
    return join(sep, Arrays.asList(arr));
  }

  public static String join(String sep, Collection<String> c)
  {
    final StringBuilder sb = new StringBuilder();
    for (final String s : c)
      sb.append(s).append(sb.length() > 0 ? sep : "");
    return sb.toString();
  }

  //

  public static String utf8(byte[] bin)
  {
    return (bin == null) ? null : new String(bin, Charset.forName("UTF-8"));
  }

  public static byte[] utf8(String s)
  {
    return (s == null) ? null : s.getBytes(Charset.forName("UTF-8"));
  }

  //

  public static String trimToNull(String s)
  {
    if (s == null)
      return null;
    s = s.trim();
    return s.isEmpty() ? null : s;
  }

  public static String trimToEmpty(String s)
  {
    s = Text.trimToNull(s);
    return (s == null) ? "" : s;
  }

  //

  public static String before(String s, String find, String fallback)
  {
    final int io = s.indexOf(find);
    if (io == -1)
      return fallback;
    return s.substring(0, io);
  }

  public static String after(String s, String find, String fallback)
  {
    final int io = s.indexOf(find);
    if (io == -1)
      return fallback;
    return s.substring(io + find.length());
  }

  //

  public static String beforeLast(String s, String find, String fallback)
  {
    final int io = s.lastIndexOf(find);
    if (io == -1)
      return fallback;
    return s.substring(0, io);
  }

  public static String afterLast(String s, String find, String fallback)
  {
    final int io = s.lastIndexOf(find);
    if (io == -1)
      return fallback;
    return s.substring(io + find.length());
  }

  //

  public static String before(String s, String find)
  {
    return before(s, find, null);
  }

  public static String after(String s, String find)
  {
    return after(s, find, null);
  }

  public static String beforeLast(String s, String find)
  {
    return beforeLast(s, find, null);
  }

  public static String afterLast(String s, String find)
  {
    return afterLast(s, find, null);
  }

  //

  public static String beforeIfAny(String s, String find)
  {
    return before(s, find, s);
  }

  public static String afterIfAny(String s, String find)
  {
    return after(s, find, s);
  }

  public static String beforeLastIfAny(String s, String find)
  {
    return beforeLast(s, find, s);
  }

  public static String afterLastIfAny(String s, String find)
  {
    return afterLast(s, find, s);
  }
}
