package nl.west.rme.common;

import nl.west.rme.common.util.RegexRules;
import nl.west.rme.common.util.Text;
import javax.servlet.FilterConfig;


public class FilterParamUtils
{
  public static int parseInt(FilterConfig config, String prop, int orElse)
  {
    final String val = Text.trimToNull(config.getInitParameter(prop));
    if (val == null)
      return orElse;

    try {
      return Integer.parseInt(val);
    }
    catch (final NumberFormatException exc) {
      throw new IllegalStateException("prop not an int: " + prop + " ('" + val
          + "')", exc);
    }
  }

  @SuppressWarnings("unchecked")
  public static <T> T instantiateFQCN(FilterConfig config, String prop)
  {
    final String fqcn = Text.trimToNull(config.getInitParameter(prop));
    if (fqcn == null)
      return null;

    try {
      return (T) Class.forName(fqcn).newInstance();
    }
    catch (InstantiationException | IllegalAccessException
        | ClassNotFoundException e) {
      throw new IllegalStateException(prop + " unusable");
    }
  }

  public static RegexRules uriRules(
      FilterConfig config, String prop, String name)
  {
    final String rules = Text.trimToNull(config.getInitParameter(prop));
    if (rules == null)
      return null;

    final RegexRules filter = new RegexRules(name);
    filter.parseRules(rules);
    return filter;
  }
}
