/**
 *
 */
package nl.west.rme.common.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;


public class CollectionUtils
{
  public static <T> List<List<T>> chunk(Collection<T> entries, int chunkSize)
  {
    final List<List<T>> chunkList = new ArrayList<>();
    int i = 0;
    for (final T entry : entries) {
      if (i % chunkSize == 0)
        chunkList.add(new ArrayList<T>());
      chunkList.get(i++ / chunkSize).add(entry);
    }
    return chunkList;
  }

  public static <T> T getLast(List<? extends T> array)
  {
    final int size = array.size();
    if (size == 0) {
      return null;
    }
    else {
      return array.get(size - 1);
    }
  }

  public static <T> T getLast(T[] array)
  {
    return getLast(Arrays.asList(array));
  }

  @SuppressWarnings("unchecked")
  public static <T, S extends Collection<T>> S newCollectionByType(S values)
  {
    if (values instanceof Set)
      return (S) new HashSet<>();
    return (S) new ArrayList<>();
  }

  public static <K, V extends Comparable<? super V>> void sortMapByValue(
      LinkedHashMap<K, V> map)
  {
    final List<Entry<K, V>> list = new ArrayList<Entry<K, V>>(map.entrySet());
    map.clear();
    Collections.sort(list, new Comparator<Entry<K, V>>()
    {
      @Override
      public int compare(Entry<K, V> o1, Entry<K, V> o2)
      {
        return o1.getValue().compareTo(o2.getValue());
      }
    });
    for (final Entry<K, V> entry : list) {
      map.put(entry.getKey(), entry.getValue());
    }
  }

  public static <K, V> void sortMapByValue(
      LinkedHashMap<K, V> map, final Comparator<V> comparator)
  {
    final List<Entry<K, V>> list = new ArrayList<Entry<K, V>>(map.entrySet());
    map.clear();
    Collections.sort(list, new Comparator<Entry<K, V>>()
    {
      @Override
      public int compare(Entry<K, V> o1, Entry<K, V> o2)
      {
        return comparator.compare(o1.getValue(), o2.getValue());
      }
    });
    for (final Entry<K, V> entry : list) {
      map.put(entry.getKey(), entry.getValue());
    }
  }

  public static <K extends Comparable<? super K>, V> void sortMapByKey(
      LinkedHashMap<K, V> map)
  {
    final List<Entry<K, V>> list = new ArrayList<Entry<K, V>>(map.entrySet());
    map.clear();
    Collections.sort(list, new Comparator<Entry<K, V>>()
    {
      @Override
      public int compare(Entry<K, V> o1, Entry<K, V> o2)
      {
        return o1.getKey().compareTo(o2.getKey());
      }
    });
    for (final Entry<K, V> entry : list) {
      map.put(entry.getKey(), entry.getValue());
    }
  }

  public static <K, V> void sortMapByKey(
      LinkedHashMap<K, V> map, final Comparator<K> comparator)
  {
    final List<Entry<K, V>> list = new ArrayList<Entry<K, V>>(map.entrySet());
    map.clear();
    Collections.sort(list, new Comparator<Entry<K, V>>()
    {
      @Override
      public int compare(Entry<K, V> o1, Entry<K, V> o2)
      {
        return comparator.compare(o1.getKey(), o2.getKey());
      }
    });
    for (final Entry<K, V> entry : list) {
      map.put(entry.getKey(), entry.getValue());
    }
  }

  public static <K, V> Map<K, V> combine(
      List<? extends K> keys, List<? extends V> values)
  {
    if (keys.size() != values.size()) {
      throw new IllegalArgumentException(
          "Cannot combine lists with dissimilar sizes");
    }

    final Map<K, V> map = new LinkedHashMap<K, V>(keys.size());
    for (int i = 0; i < keys.size(); i++) {
      map.put(keys.get(i), values.get(i));
    }
    return map;
  }

  public static interface InitialValue<K, V>
  {
    public V get(K k);
  }

  @SuppressWarnings("serial")
  public static <K, V> Map<K, V> initialValueMap(final InitialValue<K, V> iv)
  {
    return new HashMap<K, V>()
    {
      @Override
      public V get(Object key)
      {
        @SuppressWarnings("unchecked")
        final K typedKey = (K) key;

        V result = super.get(key);
        if (result == null && !super.containsKey(key))
          super.put(typedKey, result = iv.get(typedKey));
        return result;
      }
    };
  }
}
