package nl.west.rme.common.util;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;


public class Compare
{
  public static void assertEq(Object a, Object b)
  {
    if (!Compare.equals(a, b)) {
      throw new IllegalStateException("'" + a + "' vs '" + b + "'");
    }
  }

  public static boolean equals(Object a, Object b)
  {
    if (a == b)
      return true; // either both null, or the same object
    if ((a == null) ^ (b == null))
      return false; // one of them is null
    return a.equals(b);
  }

  public static boolean equalsIgnoreCase(String a, String b)
  {
    if (a == b)
      return true; // either both null, or the same object
    if ((a == null) ^ (b == null))
      return false; // one of them is null
    return a.equalsIgnoreCase(b);
  }

  @SuppressWarnings({ "unchecked", "rawtypes" })
  public static int compare(Object a, Object b)
  {
    if (a != null && !(a instanceof Comparable<?>))
      throw new IllegalStateException();
    if (b != null && !(b instanceof Comparable<?>))
      throw new IllegalStateException();

    if (equals(a, b))
      return 0;

    if (a == null)
      return -1;
    if (b == null)
      return +1;

    return ((Comparable) a).compareTo(b);
  }

  public static int compareIgnoreCase(String a, String b)
  {
    if (a != null)
      a = a.toLowerCase();
    if (b != null)
      b = b.toLowerCase();
    return compare(a, b);
  }

  private static final Set<String> operators = new HashSet<>();
  static {
    operators.add("=");
    operators.add("==");
    operators.add("!=");
    operators.add("<");
    operators.add(">");
    operators.add("<=");
    operators.add(">=");
    operators.add("LIKE");
  }

  public static Set<String> supportedOperators()
  {
    return Collections.unmodifiableSet(operators);
  }

  public static boolean operator(Object a, String cmp, Object b)
  {
    switch (cmp) {
      case "=":
      case "==":
        return Compare.equals(a, b);
      case "!=":
        return !Compare.equals(a, b);

      case ">":
        return Compare.compare(a, b) > 0;
      case "<":
        return Compare.compare(a, b) < 0;
      case ">=":
        return Compare.compare(a, b) >= 0;
      case "<=":
        return Compare.compare(a, b) <= 0;

      case "LIKE": {
        if (a != null && !(a instanceof String))
          throw new IllegalStateException();
        if (b == null)
          throw new IllegalStateException();
        if (a == null)
          return false;

        final String search = (String) a;
        final String pattern = (String) b;

        final int io = pattern.indexOf('%');
        if (io == -1)
          throw new IllegalStateException("must contain '%'");
        if (io == pattern.lastIndexOf('%'))
          return search.startsWith(pattern.substring(0, io)) && //
              search.endsWith(pattern.substring(io + 1, pattern.length()));
        if (io == 0 && pattern.lastIndexOf('%') == pattern.length() - 1)
          return search.contains(pattern.substring(1, pattern.length() - 1));
        throw new UnsupportedOperationException("unsupported pattern: '"
            + pattern + "'");
      }

      default:
        throw new IllegalStateException("unknown comparator: '" + cmp + "'");
    }
  }
}
