package nl.west.rme.common.util;

import java.util.Iterator;
import java.util.NoSuchElementException;


public class IterUtils
{
  public static interface IterStream<T>
  {
    /**
     * The stream ends when null is returned
     *
     * @return
     */
    public T fetch();

    public void remove(T obj);
  }

  public static abstract class IterFetchStream<T> implements IterStream<T>
  {
    @Override
    public final void remove(T obj)
    {
      throw new UnsupportedOperationException();
    }
  }

  public static <T> Iterator<T> iterator(final IterStream<T> stream)
  {
    return new Iterator<T>()
    {
      private T curObject = null;
      private T nxtObject = null;
      private boolean active = true;

      @Override
      public boolean hasNext()
      {
        if (active && nxtObject == null) {
          nxtObject = stream.fetch();
          active = nxtObject != null;
        }
        return active;
      }

      @Override
      public T next()
      {
        if (!hasNext()) {
          curObject = null;
          throw new NoSuchElementException();
        }
        curObject = nxtObject;
        nxtObject = null;

        return curObject;
      }

      @Override
      public void remove()
      {
        if (curObject == null) {
          throw new IllegalStateException();
        }
        stream.remove(curObject);
        curObject = null;
      }
    };
  }

  public static <T> Iterable<T> iterable(final IterStream<T> stream)
  {
    return iterable(iterator(stream));
  }

  public static <T> Iterable<T> iterable(final Iterator<T> iter)
  {
    return new Iterable<T>()
    {
      @Override
      public Iterator<T> iterator()
      {
        return iter;
      }
    };
  }
}
