package nl.west.rme.common.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;


public class MapSet<K, V>
{
  private final Map<K, Set<V>> data;

  public MapSet()
  {
    this.data = this.createParent();
  }

  public Set<K> keys()
  {
    return data.keySet();
  }

  //

  public boolean add(K key, V value)
  {
    Set<V> child = data.get(key);
    if (child == null)
      data.put(key, child = this.createChild());
    return child.add(value);
  }

  public boolean contains(K key, V value)
  {
    final Set<V> child = data.get(key);
    return (child == null) ? false : child.contains(value);
  }

  public boolean remove(K key, V value)
  {
    final Set<V> child = data.get(key);
    final boolean isRemoved = (child == null) ? false : child.remove(value);
    if (isRemoved && child.isEmpty())
      data.remove(key);
    return isRemoved;
  }

  //

  public Set<V> currentChild(K key)
  {
    return data.get(key);
  }

  public Set<V> ensureChild(K key)
  {
    Set<V> child = data.get(key);
    if (child == null)
      data.put(key, child = this.createChild());
    return child;
  }

  //

  public MapSet<V, K> transpose()
  {
    final MapSet<V, K> transposed = new MapSet<V, K>();
    for (final Entry<K, Set<V>> entry : data.entrySet())
      for (final V val : entry.getValue())
        transposed.add(val, entry.getKey());
    return transposed;
  }

  //

  protected Map<K, Set<V>> createParent()
  {
    return new HashMap<K, Set<V>>();
  }

  protected Set<V> createChild()
  {
    return new HashSet<V>();
  }
}
