package nl.west.rme.common.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.Random;


public class Text
{
  public static String ascii(byte[] data)
  {
    if (data == null)
      return null;
    final char[] ca = new char[data.length];
    for (int i = 0; i < ca.length; i++)
      ca[i] = (char) data[i];
    return new String(ca);
  }

  public static byte[] ascii(String data)
  {
    if (data == null)
      return null;
    final byte[] ba = new byte[data.length()];
    for (int i = 0; i < ba.length; i++)
      if (data.charAt(i) <= 0x00ff)
        ba[i] = (byte) data.charAt(i);
      else
        throw new IllegalStateException();
    return ba;
  }

  //

  private static final Charset utf8_charset = Charset.forName("utf-8");

  public static String utf8(byte[] data)
  {
    if (data == null)
      return null;
    return new String(data, utf8_charset);
  }

  public static byte[] utf8(String data)
  {
    if (data == null)
      return null;
    return data.getBytes(utf8_charset);
  }

  //

  public static String utf8urlencode(String value)
  {
    if (value == null) {
      return null;
    }
    try {
      return URLEncoder.encode(value, "UTF-8");
    }
    catch (final UnsupportedEncodingException exc) {
      throw new IllegalStateException(exc);
    }
  }

  public static String utf8urldecode(String value)
  {
    if (value == null) {
      return null;
    }
    try {
      return URLDecoder.decode(value, "UTF-8");
    }
    catch (final UnsupportedEncodingException exc) {
      throw new IllegalStateException(exc);
    }
  }

  //

  public static String normalizeLines(String s)
  {
    return s.replace("\r\n", "\n").replace("\r", "\n");
  }

  public static String[] splitOnLines(String s)
  {
    return normalizeLines(s).split("\n");
  }

  public static final String LOWERCASE_ALPHABET = "abcdefghijklmnopqrstuvwxyz";
  public static final String UPPERCASE_ALPHABET = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

  public static String randomText(int length, String chars, Random random)
  {
    final char[] array = new char[length];
    for (int i = 0; i < array.length; i++)
      array[i] = chars.charAt(random.nextInt(chars.length()));
    return new String(array);
  }

  public static String pronounceableRandomSentence(int length)
  {
    final Random rndm = new Random();

    final StringBuilder sb = new StringBuilder();
    while (sb.length() < length) {
      int len = 3 + rndm.nextInt(3) * (1 + rndm.nextInt(3));
      if (sb.length() + len >= length)
        len = length - sb.length() - 1;
      if (len < 3)
        break;

      final String word = pronounceableRandomWord(len);
      if (sb.length() != 0)
        sb.append(' ');
      sb.append(word);
    }

    if (sb.length() == 0)
      return "";

    String sentence = sb.toString();
    sentence = sentence.substring(0, 1).toUpperCase() + sentence.substring(1);
    return sentence;
  }

  public static String pronounceableRandomWord(int length)
  {
    final String vowels = "aaaeeeiouu";
    final String vowels2 = "aeou";
    final String consonants = "bbcddffgghklmnpqrsstvy";
    final String consonants2 = "bdmns";

    final double chanceStartWithVowel = 0.33;
    final double chanceVowelPair = 0.25;
    final double chanceVowelDuplicate = 0.25;
    final double chanceConsonantDuplicate = 0.15;

    final Random rndm = new Random();

    boolean wasLastVowel = false;

    final StringBuilder word = new StringBuilder();
    while (word.length() < length) {
      if (word.length() == 0) {
        wasLastVowel = (rndm.nextDouble() < chanceStartWithVowel);
        if (wasLastVowel)
          word.append(vowels.charAt(rndm.nextInt(vowels.length())));
        else
          word.append(consonants.charAt(rndm.nextInt(consonants.length())));
      }
      else {
        final boolean isNextVowel = !wasLastVowel;
        char c;
        if (isNextVowel)
          c = vowels.charAt(rndm.nextInt(vowels.length()));
        else
          c = consonants.charAt(rndm.nextInt(consonants.length()));
        word.append(c);

        if (word.length() < length - 2 && isNextVowel) {
          if (rndm.nextDouble() < chanceVowelPair && vowels2.indexOf(c) != -1)
            word.append(c);
        }

        if (word.length() < length - 2) {
          final double chanceDuplicate =
            (isNextVowel ? chanceVowelDuplicate : chanceConsonantDuplicate);
          final String chars2 = (isNextVowel ? vowels2 : consonants2);
          if (rndm.nextDouble() < chanceDuplicate && chars2.indexOf(c) != -1)
            word.append(c);
        }

        wasLastVowel = isNextVowel;
      }
    }
    return word.toString();
  }

  //
  private static final char[] hex_lookup_table = "0123456789abcdef"
      .toCharArray();

  public static String toHex(byte[] data)
  {
    final char[] hex = new char[data.length * 2];
    for (int i = 0, p = 0; i < data.length; i++) {
      hex[p++] = hex_lookup_table[(data[i] >> 4) & 0xF];
      hex[p++] = hex_lookup_table[(data[i] >> 0) & 0xF];
    }
    return new String(hex);
  }

  public static byte[] fromHex(String hex)
  {
    if (hex.length() % 2 != 0)
      throw new IllegalStateException();

    final byte[] raw = new byte[hex.length() >> 1];
    for (int i = 0; i < hex.length(); i++) {
      final char c = hex.charAt(i);

      int v;
      if (c >= '0' && c <= '9')
        v = (c - '0');
      else if (c >= 'a' && c <= 'f')
        v = (c - 'a') + 10;
      else if (c >= 'A' && c <= 'F')
        v = (c - 'A') + 10;
      else
        throw new IllegalStateException("char: " + c);

      raw[i >> 1] |= v << ((~i & 1) * 4);
    }
    return raw;
  }

  //

  public static String before(String value, char find)
  {
    return before(value, find, null);
  }

  public static String before(String value, char find, String orElse)
  {
    final int io = value.indexOf(find);
    return (io == -1) ? orElse : value.substring(0, io);
  }

  public static String before(String value, String find)
  {
    return before(value, find, null);
  }

  public static String before(String value, String find, String orElse)
  {
    final int io = value.indexOf(find);
    return (io == -1) ? orElse : value.substring(0, io);
  }

  public static String after(String value, char find)
  {
    return after(value, find, null);
  }

  public static String after(String value, char find, String orElse)
  {
    final int io = value.indexOf(find);
    return (io == -1) ? orElse : value.substring(io + 1);
  }

  public static String after(String value, String find)
  {
    return after(value, find, null);
  }

  public static String after(String value, String find, String orElse)
  {
    final int io = value.indexOf(find);
    return (io == -1) ? orElse : value.substring(io + find.length());
  }

  public static String between(String value, char start, char end)
  {
    return Text.between(value, start, end, null);
  }

  public static String between(String value, char start, char end, String orElse)
  {
    final String after = Text.after(value, start, null);
    return (after == null) ? orElse : Text.before(after, end, orElse);
  }

  public static String between(String value, String start, String end)
  {
    return Text.between(value, start, end, null);
  }

  public static String between(
      String value, String start, String end, String orElse)
  {
    final String after = Text.after(value, start, null);
    return (after == null) ? orElse : Text.before(after, end, orElse);
  }

  public static int count(String value, char c)
  {
    int a = 0;
    for (int i = 0; i < value.length(); i++)
      if (value.charAt(c) == c)
        a++;
    return a;
  }

  public static String removeLast(String value, char find)
  {
    final int indexOf = value.lastIndexOf(find);
    if (indexOf != -1) {
      final String before = value.substring(0, indexOf);
      final String after = value.substring(indexOf + 1);
      value = before + after;
    }
    return value;
  }

  public static String trimToNull(String s)
  {
    if (s == null)
      return null;
    s = s.trim();
    if (s.isEmpty())
      return null;
    return s;
  }

  public static String trimToEmpty(String s)
  {
    return ((s = Text.trimToNull(s)) == null) ? "" : s;
  }

  // ----

  public static String dashesToCapitalize(String in)
  {
    final StringBuilder sb = new StringBuilder();
    for (final String s : in.split("-"))
      if (!s.isEmpty())
        sb.append(s.substring(0, 1).toUpperCase()).append(
            s.substring(1).toLowerCase());
    return sb.toString();
  }

  public static String dashesToCamelCase(String in)
  {
    final StringBuilder sb = new StringBuilder();
    for (final String s : in.split("-"))
      if (!s.isEmpty())
        if (sb.length() == 0)
          sb.append(s);
        else
          sb.append(s.substring(0, 1).toUpperCase()).append(
              s.substring(1).toLowerCase());
    return sb.toString();
  }

  public static String capitalsToDashes(String in)
  {
    final StringBuilder sb = new StringBuilder();
    for (int i = 0; i < in.length(); i++) {
      char c = in.charAt(i);
      if (Character.isUpperCase(c)) {
        if (sb.length() > 0)
          sb.append('-');
        c = Character.toLowerCase(c);
      }
      sb.append(c);
    }
    return sb.toString();
  }
}
