/*
 * NAME
 *    $HeadURL$
 * DESCRIPTION
 *    [given below in javadoc format]
 * COPYRIGHT
 *    (c) 2015 West Consulting BV
 * VERSION ID
 *    $Id$
 */
package nl.west.rme.common.util;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class WebContextThreadLocal<V> {
	private final Map<Thread, V> thread2value = new HashMap<>();
	private boolean isActive = true;

	protected V initialValue() {
		return null;
	}

	protected void onDestroy(Thread t, V value) {

	}

	public final void set(V value) {
		final Thread key = Thread.currentThread();
		synchronized (thread2value) {
			if (!isActive)
				throw new IllegalStateException();

			thread2value.put(key, value);
		}
	}

	public final V get() {
		final Thread key = Thread.currentThread();
		synchronized (thread2value) {
			if (!isActive)
				throw new IllegalStateException();

			if (thread2value.containsKey(key)) {
				return thread2value.get(key);
			}
			final V value = this.initialValue();
			thread2value.put(key, value);
			return value;
		}
	}

	public final V remove() {
		final Thread key = Thread.currentThread();
		synchronized (thread2value) {
			if (!isActive)
				throw new IllegalStateException();

			return thread2value.remove(key);
		}
	}

	public final void gc() {
		final Map<Thread, V> copy = new HashMap<>(thread2value);

		synchronized (thread2value) {
			for (final Entry<Thread, V> entry : thread2value.entrySet()) {
				if (!entry.getKey().isAlive()) {
					copy.put(entry.getKey(), entry.getValue());
				}
			}
			for (final Thread key : copy.keySet())
				thread2value.remove(key);
		}

		this.doDestroy(copy);
	}

	public final void destroy() {
		Map<Thread, V> copy;

		synchronized (thread2value) {
			if (!isActive)
				throw new IllegalStateException();
			isActive = false;

			copy = new HashMap<>(thread2value);
			thread2value.clear();
		}

		this.doDestroy(copy);
	}

	private void doDestroy(Map<Thread, V> copy) {
		for (final Entry<Thread, V> entry : copy.entrySet()) {
			this.onDestroy(entry.getKey(), entry.getValue());
		}
	}
}
